package com.ruoyi.bst.order.service.impl;

import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderBO;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.mapper.OrderMapper;
import com.ruoyi.bst.order.service.IOrderService;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.orderProd.domain.OrderProd;
import com.ruoyi.bst.orderProd.domain.OrderProdBO;
import com.ruoyi.bst.orderProd.domain.OrderProdVO;
import com.ruoyi.bst.orderProd.service.IOrderProdService;
import com.ruoyi.bst.prodProcess.domain.ProdProcessBO;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;

/**
 * 生产订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-12-16
 */
@Service
public class OrderServiceImpl implements IOrderService
{
    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderValidator orderValidator;

    @Autowired
    private IOrderProdService orderProdService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询生产订单
     *
     * @param id 生产订单主键
     * @return 生产订单
     */
    @Override
    public OrderVO selectOrderById(Long id)
    {
        return orderMapper.selectOrderById(id);
    }

    /**
     * 查询生产订单列表
     *
     * @param order 生产订单
     * @return 生产订单
     */
    @Override
    public List<OrderVO> selectOrderList(OrderQuery order)
    {
        return orderMapper.selectOrderList(order);
    }

    /**
     * 新增生产订单
     *
     * @param order 生产订单
     * @return 结果
     */
    @Override
    public int insertOrder(Order order)
    {
        order.setOrderNo(String.valueOf(SnowFlakeUtil.newId()));
        order.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertOrder(order);
    }

    /**
     * 修改生产订单
     *
     * @param order 生产订单
     * @return 结果
     */
    @Override
    public int updateOrder(Order order)
    {
        return orderMapper.updateOrder(order);
    }

    /**
     * 批量删除生产订单
     *
     * @param ids 需要删除的生产订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderByIds(Long[] ids)
    {
        return orderMapper.deleteOrderByIds(ids);
    }

    /**
     * 删除生产订单信息
     *
     * @param id 生产订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderById(Long id)
    {
        return orderMapper.deleteOrderById(id);
    }

    @Override
    public int addOrder(OrderBO order) {
        orderValidator.checkBefore(order);

        Integer result = transactionTemplate.execute(status -> {
            int insert = this.insertOrder(order);

            if (insert == 1) {
                // 批量更新产品明细
                this.batchUpdateProdList(order);

                // 批量更新工序明细
                this.batchUpdateProcessList(order);
            }

            return insert;
        });

        return result == null ? 0 : result;
    }

    private int batchUpdateProcessList(OrderBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProdList())) {
            return 0;
        }
        // 关联
        for (OrderProdBO prod : bo.getProdList()) {
            for (ProdProcessBO process : prod.getProcessList()) {
                process.setOrderProdId(prod.getId());
            }
        }

    }

    private int batchUpdateProdList(OrderBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProdList())) {
            return 0;
        }
        // 关联订单ID

        for (OrderProd prod : bo.getProdList()) {
            prod.setOrderId(bo.getId());
        }

        List<OrderProdVO> oldList = orderProdService.selectByOrderId(bo.getId());

        // 分离出新增、修改、删除的列表
        DiffListVO<OrderProdBO, OrderProdVO> diff = CollectionUtils.convertToDiffList(bo.getProdList(), oldList, OrderProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int batchInsert = orderProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(batchInsert != diff.getAddCount(), "新增报表产量明细失败");
            result += batchInsert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = orderProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改报表产量明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = orderProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除报表产量明细失败");
            result += del;
        }

        return result;
    }
}
